
#
# CHECK_VERSION(FOUND-VERSION, MINIMUM-VERSION)
#
# Handles versions of the forms x.y and x.y.z where z can be lowercase letters
# and numbers. If z has letters, everything from the first letter to the end of
# the string is thrown out.
#
# Sets CHECK_VERSION_RESULT to 0 if the found- and minimum-versions are the
# same; to 1 if the found-version is later than the minimum-version; to -1 if
# the found-version is earlier than the minimum-version.
#
AC_DEFUN([CHECK_VERSION], [
	found_version=$1
	min_version=$2

	# Determine if we're of the form x.y or x.y.z
	form=`echo $min_version | sed 's/[[0-9]]*\.[[0-9]]*\.[[0-9a-z]]*/DOT_DOT/'`

	# Parse version strings
	if test "$form" = "DOT_DOT"; then
		found_major_version=`echo $found_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9a-z]]*\)/\1/'`
		found_minor_version=`echo $found_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9a-z]]*\)/\2/'`
		found_micro_version=`echo $found_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9]]*\)[[0-9a-z]]*/\3/'`
		min_major_version=`echo $min_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9a-z]]*\)/\1/'`
		min_minor_version=`echo $min_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9a-z]]*\)/\2/'`
		min_micro_version=`echo $min_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)\.\([[0-9]]*\)[[0-9a-z]]*/\3/'`
	else
		found_major_version=`echo $found_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)/\1/'`
		found_minor_version=`echo $found_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)/\2/'`
		found_micro_version=0
		min_major_version=`echo $min_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)/\1/'`
		min_minor_version=`echo $min_version | \
			sed 's/\([[0-9]]*\)\.\([[0-9]]*\)/\2/'`
		min_micro_version=0
	fi

	# Compare versions
	if test $found_major_version -eq $min_major_version && \
	   test $found_minor_version -eq $min_minor_version && \
	   test $found_micro_version -eq $min_micro_version; then
		version_result=0
	else
		version_result=-1
		if test $found_major_version -gt $min_major_version; then
			version_result=1
		elif test $found_major_version -eq $min_major_version && \
			 test $found_minor_version -gt $min_minor_version; then
			version_result=1
		elif test $found_major_version -eq $min_major_version && \
			 test $found_minor_version -eq $min_minor_version && \
			 test $found_micro_version -ge $min_micro_version; then
			version_result=1
		fi
	fi

	AC_SUBST(CHECK_VERSION_RESULT, $version_result)
])


#
# CHECK_DESIRED_VERSION(LIBRARY-NAME, DESIRED-VERSION, FOUND-VERSION)
#
AC_DEFUN([CHECK_DESIRED_VERSION], [
	library_name=$1
	desired_version=$2
	found_version=$3

	if test "$STRICT_LIBRARY_CHECK" = "yes"; then
		AC_MSG_CHECKING(for $library_name version == $desired_version)
	else
		AC_MSG_CHECKING(for $library_name version >= $desired_version)
	fi

	CHECK_VERSION($found_version, $desired_version)
	version_check=$CHECK_VERSION_RESULT
	if test $version_check -lt 0; then
		AC_MSG_RESULT(no)
		echo "*** $library_name version $desired_version or later is required, but version $found_version "
            echo "*** was found instead. Please update your $library_name to version $desired_version."
		exit -1
	elif test $version_check -gt 0; then
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			AC_MSG_RESULT(no)
		else
			AC_MSG_RESULT(yes)
		fi
		echo "### $library_name version $found_version was found. That version may work, but the"
		echo "### officially supported version is $desired_version."
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			echo "*** Strict library check failed. Either install the officially supported"
			echo "*** version, or don't use the --enable-strict-library-check option."
			exit -1
		fi
	else
		AC_MSG_RESULT(yes)
	fi
])


#
# CHECK_DESIRED_VERSION(LIBRARY-NAME, DESIRED-VERSION, FOUND-VERSION)
#
AC_DEFUN([CHECK_DESIRED_VERSION], [
	library_name=$1
	desired_version=$2
	found_version=$3

	if test "$STRICT_LIBRARY_CHECK" = "yes"; then
		AC_MSG_CHECKING(for $library_name version == $desired_version)
	else
		AC_MSG_CHECKING(for $library_name version >= $desired_version)
	fi

	CHECK_VERSION($found_version, $desired_version)
	version_check=$CHECK_VERSION_RESULT
	if test $version_check -lt 0; then
		AC_MSG_RESULT(no)
		echo "*** $library_name version $desired_version or later is required, but version $found_version "
            echo "*** was found instead. Please update your $library_name to version $desired_version."
		exit -1
	elif test $version_check -gt 0; then
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			AC_MSG_RESULT(no)
		else
			AC_MSG_RESULT(yes)
		fi
		echo "### $library_name version $found_version was found. That version may work, but the"
		echo "### officially supported version is $desired_version."
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			echo "*** Strict library check failed. Either install the officially supported"
			echo "*** version, or don't use the --enable-strict-library-check option."
			exit -1
		fi
	else
		AC_MSG_RESULT(yes)
	fi
])


#
# CHECK_FREEZEPYTHON(DESIRED-VERSION)
#
AC_DEFUN([CHECK_FREEZEPYTHON], [
	desired_version=$1

	if test -n "$FREEZEPYTHON"; then

		# Check version
		found_version=`$FREEZEPYTHON --version`
		found_version=`echo $found_version | \
			sed 's/FreezePython \([[0-9]]*.[[0-9]]*.[[0-9]]*\).*/\1/'`
		CHECK_VERSION($found_version, $desired_version)
		version_check=$CHECK_VERSION_RESULT
		if test $version_check -lt 0; then
			echo "*** FreezePython version $desired_version or later is required, but version $found_version "
			echo "*** was found instead. Either update your FreezePython, or don't use the"
			echo "*** --with-freezepython-prefix option."
			exit -1
		elif test $version_check -gt 0; then
			echo "### FreezePython version $found_version was found. That version may work, but the"
			echo "### officially supported version is $desired_version."
			if test "$STRICT_LIBRARY_CHECK" = "yes"; then
				echo "*** Strict library check failed. Either install the officially supported"
				echo "*** version, or don't use the --enable-strict-library-check option."
				exit -1
			fi
		fi
	else
		echo "### We're not building an executable distribution nor freezing the app."
		echo "### The --with-freezepython-prefix option was not specified."
	fi
])


#
# CHECK_QT(DESIRED-QT-VERSION)
#
AC_DEFUN([CHECK_QT], [
	desired_version=$1

	QT4_DO_IT_ALL
	if test "$STRICT_LIBRARY_CHECK" = "yes"; then
		AC_MSG_CHECKING(for Qt version == $desired_version)
	else
		AC_MSG_CHECKING(for Qt version >= $desired_version)
	fi
	found_version=${QT4_VERSION}

	CHECK_VERSION($found_version, $desired_version)
	version_check=$CHECK_VERSION_RESULT
	if test $version_check -lt 0; then
		AC_MSG_RESULT(no)
		echo "*** Qt version $desired_version or later is required, but version $found_version "
            echo "*** was found instead. Please update your Qt to version $desired_version."
		exit -1
	elif test $version_check -gt 0; then
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			AC_MSG_RESULT(no)
		else
			AC_MSG_RESULT(yes)
		fi
		echo "### Qt version $found_version was found. That version may work, but the"
		echo "### officially supported version is $desired_version."
		if test "$STRICT_LIBRARY_CHECK" = "yes"; then
			echo "*** Strict library check failed. Either install the officially supported"
			echo "*** version, or don't use the --enable-strict-library-check option."
			exit -1
		fi
	else
		AC_MSG_RESULT(yes)
	fi
])

#
# The below was acquired from http://www.lyx.org/
# No copyright information was found at the time.
#

dnl check a particular libname
AC_DEFUN([QT4_TRY_LINK],
[
	SAVE_LIBS="$LIBS"
	LIBS="$LIBS $1"
	AC_TRY_LINK([
	#include <qglobal.h>
	#include <qstring.h>
		],
	[
	QString s("mangle_failure");
	#if (QT_VERSION < 400)
	break_me_(\\\);
	#endif
	],
	qt4_cv_libname=$1,
	)
	LIBS="$SAVE_LIBS"
])

dnl check we can do a compile
AC_DEFUN([QT4_CHECK_COMPILE],
[
	AC_MSG_CHECKING([for Qt 4 library name])

	AC_CACHE_VAL(qt4_cv_libname,
	[
		AC_LANG_CPLUSPLUS
		SAVE_CXXFLAGS=$CXXFLAGS
		CXXFLAGS="$CXXFLAGS $QT4_INCLUDES $QT4_LDFLAGS"
		for libname in -lQtCore -lQtCore4
		do
			QT4_TRY_LINK($libname)
			if test -n "$qt4_cv_libname"; then
				QT4_CORE_LIB="$qt4_cv_libname"
				break;
			fi
		done
		qt4_cv_libname=
		for libname in '-lQtCore -lQtGui' \
		               '-lQtCore4 -lQtGui4'
		do
			QT4_TRY_LINK($libname)
			if test -n "$qt4_cv_libname"; then
				break;
			fi
		done
		CXXFLAGS=$SAVE_CXXFLAGS
	])

	if test -z "$qt4_cv_libname"; then
		AC_MSG_RESULT([failed])
		if test "$FATAL" = 1 ; then
			AC_MSG_ERROR([Cannot compile a simple Qt 4 executable. Check you have the right \$QT4DIR !])
		fi
	else
		AC_MSG_RESULT([$qt4_cv_libname])
	fi
])

dnl get Qt version we're using
AC_DEFUN([QT4_GET_VERSION],
[
	AC_CACHE_CHECK([Qt 4 version],lyx_cv_qt4version,
	[
		AC_LANG_CPLUSPLUS
		SAVE_CPPFLAGS=$CPPFLAGS
		CPPFLAGS="$CPPFLAGS $QT4_INCLUDES"

		cat > conftest.$ac_ext <<EOF
#line __oline__ "configure"
#include "confdefs.h"
#include <qglobal.h>
"%%%"QT_VERSION_STR"%%%"
EOF
		lyx_cv_qt4version=`(eval "$ac_cpp conftest.$ac_ext") 2>&5 | \
			grep '^"%%%"'  2>/dev/null | \
			sed -e 's/"%%%"//g' -e 's/"//g'`
		rm -f conftest.$ac_ext
		CPPFLAGS=$SAVE_CPPFLAGS
	])

	QT4_VERSION=$lyx_cv_qt4version
	AC_SUBST(QT4_VERSION)
])

dnl start here
AC_DEFUN([QT4_DO_IT_ALL],
[
	dnl this variable is precious
	AC_ARG_VAR(QT4DIR, [the place where the Qt 4 files are, e.g. /usr/lib/qt4])

	dnl Please leave this alone. I use this file in
	dnl oprofile.
	FATAL=0

	AC_ARG_WITH(qt4-dir, [  --with-qt4-dir           where the root of Qt 4 is installed ],
		[ qt4_cv_dir=`eval echo "$withval"/` ])

	AC_ARG_WITH(qt4-includes, [  --with-qt4-includes      where the Qt 4 includes are. ],
		[ qt4_cv_includes=`eval echo "$withval"` ])

	AC_ARG_WITH(qt4-libraries, [  --with-qt4-libraries     where the Qt 4 library is installed.],
		[  qt4_cv_libraries=`eval echo "$withval"` ])

	dnl pay attention to $QT4DIR unless overridden
	if test -z "$qt4_cv_dir"; then
		qt4_cv_dir=$QT4DIR
	fi

	dnl derive inc/lib if needed
	if test -n "$qt4_cv_dir"; then
		if test -z "$qt4_cv_includes"; then
			qt4_cv_includes=$qt4_cv_dir/include
		fi
		if test -z "$qt4_cv_libraries"; then
			qt4_cv_libraries=$qt4_cv_dir/lib
		fi
	fi

	dnl compute the binary dir too
	if test -n "$qt4_cv_dir"; then
		qt4_cv_bin=$qt4_cv_dir/bin
	fi

	dnl Preprocessor flags
	QT4_CPPFLAGS="-DQT_CLEAN_NAMESPACE -DQT_GENUINE_STR -DQT_NO_STL -DQT_NO_KEYWORDS"
	case ${host} in
	*mingw*) QT4_CPPFLAGS="-DQT_DLL $QT4_CPPFLAGS";;
	*cygwin*)
	if test "x$with_x" = xno ; then
		QT4_CPPFLAGS="$QT4_CPPFLAGS -DQ_CYGWIN_WIN"
	fi;;
	esac
	AC_SUBST(QT4_CPPFLAGS)

	dnl Check if it possible to do a pkg-config
	PKG_PROG_PKG_CONFIG
	if test -n "$PKG_CONFIG" ; then
		QT4_DO_PKG_CONFIG
	fi
	if test "$pkg_failed" != "no" ; then
		QT4_DO_MANUAL_CONFIG
	fi
	AC_PATH_PROGS(MOC4, [moc-qt4 moc],[],$qt4_cv_bin:$PATH)
	AC_PATH_PROGS(UIC4, [uic-qt4 uic],[],$qt4_cv_bin:$PATH)
])

AC_DEFUN([QT4_DO_PKG_CONFIG],
[
	dnl tell pkg-config to look also in $qt4_cv_dir/lib.
	save_PKG_CONFIG_PATH=$PKG_CONFIG_PATH
	if test -n "$qt4_cv_dir" ; then
	  PKG_CONFIG_PATH=$qt4_cv_dir/lib:$qt4_cv_dir/lib/pkgconfig:$PKG_CONFIG_PATH
	  export PKG_CONFIG_PATH
	fi
	PKG_CHECK_MODULES(QT4_CORE, QtCore,,[:])
	if test "$pkg_failed" = "no" ; then
		QT4_CORE_INCLUDES=$QT4_CORE_CFLAGS
		AC_SUBST(QT4_CORE_INCLUDES)
		QT4_CORE_LDFLAGS=`$PKG_CONFIG --libs-only-L QtCore`
		AC_SUBST(QT4_CORE_LDFLAGS)
		QT4_CORE_LIB=`$PKG_CONFIG --libs-only-l QtCore`
		AC_SUBST(QT4_CORE_LIB)
	fi
	PKG_CHECK_MODULES(QT4_FRONTEND, QtCore QtGui,,[:])
	if test "$pkg_failed" = "no" ; then
		QT4_INCLUDES=$QT4_FRONTEND_CFLAGS
		dnl QT4_LDFLAGS=$QT4_FRONTEND_LIBS
		QT4_LDFLAGS=`$PKG_CONFIG --libs-only-L QtCore QtGui`
		AC_SUBST(QT4_INCLUDES)
		AC_SUBST(QT4_LDFLAGS)
		QT4_VERSION=`$PKG_CONFIG --modversion QtCore`
		AC_SUBST(QT4_VERSION)
		QT4_LIB=`$PKG_CONFIG --libs-only-l QtCore QtGui`
		AC_SUBST(QT4_LIB)
		LIBS="$LIBS `$PKG_CONFIG --libs-only-other QtCore QtGui`"
	fi
	PKG_CONFIG_PATH=$save_PKG_CONFIG_PATH
])

AC_DEFUN([QT4_DO_MANUAL_CONFIG],
[
	dnl flags for compilation
	QT4_INCLUDES=
	QT4_LDFLAGS=
	QT4_CORE_INCLUDES=
	QT4_CORE_LDFLAGS=
	if test -n "$qt4_cv_includes"; then
		QT4_INCLUDES="-I$qt4_cv_includes"
		for i in Qt QtCore QtGui; do
			QT4_INCLUDES="$QT4_INCLUDES -I$qt4_cv_includes/$i"
		done
		QT4_CORE_INCLUDES="-I$qt4_cv_includes -I$qt4_cv_includes/QtCore"
	fi
	if test -n "$qt4_cv_libraries"; then
		QT4_LDFLAGS="-L$qt4_cv_libraries"
		QT4_CORE_LDFLAGS="-L$qt4_cv_libraries"
	fi
	AC_SUBST(QT4_INCLUDES)
	AC_SUBST(QT4_CORE_INCLUDES)
	AC_SUBST(QT4_LDFLAGS)
	AC_SUBST(QT4_CORE_LDFLAGS)

	QT4_CHECK_COMPILE

	QT4_LIB=$qt4_cv_libname;
	AC_SUBST(QT4_LIB)
	AC_SUBST(QT4_CORE_LIB)

	if test -n "$qt4_cv_libname"; then
		QT4_GET_VERSION
	fi
])

#
# CHECK_PYQT(DESIRED-VERSION)
#
AC_DEFUN([CHECK_PYQT], [
	desired_version=$1

	found_version=`python -c "from PyQt4 import pyqtconfig; config = pyqtconfig.Configuration(); print config.pyqt_version_str"`
	CHECK_DESIRED_VERSION([PyQt], $found_version, $found_version)
])


#
# CHECK_NUMARRAY(DESIRED-VERSION)
#
AC_DEFUN([CHECK_NUMARRAY], [
	desired_version=$1

	found_version=`python -c "from numarray import numinclude; print numinclude.version"`
	CHECK_DESIRED_VERSION([numarray], $desired_version, $found_version)
])


#
# CHECK_NUMERIC(DESIRED-VERSION)
#
AC_DEFUN([CHECK_NUMERIC], [
	desired_version=$1

	found_version=`python -c "from numeric_version import version; print version"`
	CHECK_DESIRED_VERSION([Numeric], $desired_version, $found_version)
])


#
# CHECK_NUMPY(DESIRED-VERSION)
#
AC_DEFUN([CHECK_NUMPY], [
	desired_version=$1

	found_version=`python -c "from numpy import version; print version.version"`
	CHECK_DESIRED_VERSION([NumPy], $desired_version, $found_version)
])


#
# CHECK_CTYPES(DESIRED-VERSION)
#
AC_DEFUN([CHECK_CTYPES], [
	desired_version=$1

	found_version=`python -c "import ctypes;print ctypes.__version__"`
	CHECK_DESIRED_VERSION([ctypes], $desired_version, $found_version)
])


#
# CHECK_PYOPENGL(DESIRED-VERSION)
#
AC_DEFUN([CHECK_PYOPENGL], [
	desired_version=$1

	found_version=`python -c "import OpenGL;print OpenGL.__version__"`
	CHECK_DESIRED_VERSION([PyOpenGL], $desired_version, $found_version)
])


#
# CHECK_PIL(DESIRED-VERSION)
#
AC_DEFUN([CHECK_PIL], [
	desired_version=$1

	found_version=`python -c "from PIL import Image; print Image.VERSION"`
	CHECK_DESIRED_VERSION([PIL], $desired_version, $found_version)
])


#
# CHECK_CPPUNIT(MINIMUM-VERSION)
#

AC_DEFUN([CHECK_CPPUNIT], [
  min_version=$1

  AC_ARG_WITH(cppunit-prefix,
              AC_HELP_STRING([--with-cppunit-prefix=PREFIX],
                             [find CppUnit installed under PREFIX]),
              cppunit_prefix="$withval", cppunit_prefix="")
  AC_ARG_WITH(cppunit-exec-prefix,
              AC_HELP_STRING([--with-cppunit-exec-prefix=EPREFIX],
                             [the exec prefix where CppUnit binaries were installed]),
              cppunit_exec_prefix="$withval", cppunit_exec_prefix="")

  if test -n "$cppunit_exec_prefix"; then
    AC_PATH_PROG(CPPUNIT_CONFIG_SCRIPT, cppunit-config, no,
	             "$cppunit_exec_prefix/bin")
  else
    if test -n "$cppunit_prefix"; then
      AC_PATH_PROG(CPPUNIT_CONFIG_SCRIPT, cppunit-config, no,
	               "$cppunit_prefix/bin")
	else
      AC_PATH_PROG(CPPUNIT_CONFIG_SCRIPT, cppunit-config, no)
	fi
  fi

  if test "$CPPUNIT_CONFIG_SCRIPT" = "no" ; then
    echo "*** CppUnit version $min_version or later is required to build the CppUnit test"
	echo "*** suite, but it was not found. Either install the latest CppUnit"
	echo "*** (http://cppunit.sourceforge.net/), or"
	echo "*** adjust your PATH environment variable or use the --with-cppunit-prefix or"
	echo "*** --with-cppunit-exec-prefix options so that the cppunit-config script for"
    echo "*** your installation can be found, or don't specify the --enable-cppunit-tests"
	echo "*** option."
    exit -1
  else
    #
	# Check version
	#
    AC_MSG_CHECKING(for CppUnit version >= $min_version)
    found_lib_version=`$CPPUNIT_CONFIG_SCRIPT --version`
    found_major_version=`echo $found_lib_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)[[a-z]]*/\1/'`
    found_minor_version=`echo $found_lib_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)[[a-z]]*/\2/'`
    found_micro_version=`echo $found_lib_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)[[a-z]]*/\3/'`
           
    min_major_version=`echo "$min_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    min_minor_version=`echo "$min_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    min_micro_version=`echo "$min_version" | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
       
    version_ok=no
    if test $found_major_version -gt $min_major_version; then
      version_ok=yes
    elif test $found_major_version -eq $min_major_version && \
         test $found_minor_version -gt $min_minor_version; then
      version_ok=yes
    elif test $found_major_version -eq $min_major_version && \
         test $found_minor_version -eq $min_minor_version && \
         test $found_micro_version -ge $min_micro_version; then
      version_ok=yes
    fi
  
    if test "$version_ok" = "no"; then
      AC_MSG_RESULT(no)
      echo "*** CppUnit version $min_version or later is required, but an old version"
      echo "*** was found ($found_lib_version). Please install the latest CppUnit"
      echo "*** (http://cppunit.sourceforge.net/)."
      exit -1
    else
      AC_MSG_RESULT(yes)
    fi
	
	#
	# Check libs
	#
	libs_result=`$CPPUNIT_CONFIG_SCRIPT --libs`
	if test "x$libs_result" != "x"; then
      AC_SUBST(CPPUNIT_LIBS, "$libs_result")
	fi

    #
    # Check includes
    #
	includes_result=`$CPPUNIT_CONFIG_SCRIPT --cflags`
	if test "x$includes_result" != "x"; then
      AC_SUBST(CPPUNIT_CFLAGS, "$includes_result")
	fi
  fi
])

